<?php

namespace Porth\Borderstyle\ColumnType;


    /***************************************************************
     *
     *  Copyright notice
     *
     *  (c) 2016 Dr. Dieter Porth <info@mobger.de>
     *
     *  All rights reserved
     *
     *  This script is free software; you can redistribute it and/or modify
     *  it under the terms of the GNU General Public License as published by
     *  the Free Software Foundation; either version 3 of the License, or
     *  (at your option) any later version.
     *
     *  The GNU General Public License can be found at
     *  http://www.gnu.org/copyleft/gpl.html.
     *
     *  This copyright notice MUST APPEAR in all copies of the script!
     ***************************************************************/


/**
 * DatamapProcessClass
 */
class DatamapProcessClass
{

    /**
     * build a option for an select-tag
     *
     * @param string $prefix
     * @param string $value
     * @param string $label
     * @param string $selectedValue
     * @return string
     */
    protected function buildOptionForTemplate($prefix, $value, $label, $selectedValue)
    {
        $optionValue = $prefix . $value;
        $selected = ($selectedValue === $optionValue) ? 'selected="selected"' : ''; // %4$s
        $optionString = '                        
                    <option value="%1$s" %3$s>
                        %2$s
                    </option>';
        $result = sprintf($optionString, $optionValue, $label, $selected);
        return $result;
    }


    /**
     * @param string $label set to %1$s
     * @param string $dataTable set to %2$s
     * @param string $dataType set to %3$s
     * @return string
     */
    protected function buildLabelForTemplate($label, $dataTable, $dataType)
    {
        $optionString = '
            <label class="t3js-formengine-label" style="height:0px; display:inline-block;">
                <span class="t3-help-link" href="#" data-table="%2$s" data-field="%3$s" data-loaded="false" data-html="true" 
                            data-original-title="borderstyle-theme" data-placement="auto" data-trigger="click" alt="%1$s">
                </span>
            </label>
';
        $result = sprintf($optionString, $label,$dataTable, $dataType);
        return $result;
    }

    /**
     * @param string $formName set to %1$s
     * @param string $formId set to %2$s
     * @param string $options set to %3$s
     * @return string
     */
    protected function buildSelectForTemplate($formName, $formId, $options)
    {
        $optionString = '
            <div class="t3js-formengine-field-item">
                <div class="t3-form-field-disable"></div>
                <div class="form-control-wrap">
                    <select id="%2$s" 
                            class="form-control form-control-adapt" 
                            name="%1$s" 
                            data-formengine-validation-rules="[{"type":"select","minItems":0,"maxItems":1}]"
                    >
                        %3$s
                    </select>
                </div>
            </div>
';
        $result = sprintf($optionString, $formName, $formId, $options);
//        <option value="1" data-icon="<span alt="" title=""><span class="t3js-icon icon icon-size-small icon-state-default icon-tcarecords-tx_borderstyle_domain_model_theme-default" data-identifier="tcarecords-tx_borderstyle_domain_model_theme-default"> <span class="icon-markup"> <img src="/typo3conf/ext/borderstyle/Resources/Public/Icons/tx_borderstyle_domain_model_theme.gif" width="16" height="16" /> </span> </span></span>">Test</option>
        return $result;
    }

    /**
     * @param string $lable set to %1$s
     * @param string $field set to %2$s
     * @return string
     */
    protected function wrapFieldTemplate($field)
    {

        $optionString = '
            <div class="form-group t3js-formengine-validation-marker t3js-formengine-palette-field col-md-12">
                $s
            </div>
            ';
        $result = sprintf($field);
//        <option value="1" data-icon="<span alt="" title=""><span class="t3js-icon icon icon-size-small icon-state-default icon-tcarecords-tx_borderstyle_domain_model_theme-default" data-identifier="tcarecords-tx_borderstyle_domain_model_theme-default"> <span class="icon-markup"> <img src="/typo3conf/ext/borderstyle/Resources/Public/Icons/tx_borderstyle_domain_model_theme.gif" width="16" height="16" /> </span> </span></span>">Test</option>
        return $result;
    }

    /**
     * @param array|string $settings
     * @param string $key
     * @return string
     */
    protected function parseValueInSettings($settings, $key)
    {
        $result = $key;
        if (isset($settings)) {
            if (strpos($key, '.') !== false) {
                $keys = explode('.', trim($key));
                $dummy = $settings;
                $test = true;
                foreach ($keys as $item) {
                    if ($item === 'settings') continue;
                    if (!isset($dummy[$item])) {
                        $test = false;
                        break;
                    }
                    $dummy = $dummy[$item];
                }
                if (($test) && is_string($dummy)) {
                    $result = $dummy;
                }
            } else {
                if (isset($settings[$key])) {
                    $result = $settings[$key];
                }
            }
        }
        return $result;
    }

    /**
     * @param array|null $item
     * @param string $fieldName
     * @return string
     */
    protected function constructGetterMethod($item, $fieldName) {
        $name = ((!isset($item[$fieldName])) ?
            $fieldName :
            $item[$fieldName]
        );
        $name = ((strpos($name,'_') !== false) ?
            join(array_map('ucfirst',explode('_',$name ))) :
            ucfirst($name)
        );
        return 'get'.$name;
    }


    /**
     * use a alternative field to fill an single select
     *
     * @param $fieldArray
     * @param $resource
     * @return string
     */
    public function itemsForeigntableDataChange($fieldArray, &$resource)
    {
        $parameters = $fieldArray['parameters'];

        // read Settings
        if (!isset($parameters['default']['extensionSettings'])) {
            return '';
        }
        /** @var \TYPO3\CMS\Extbase\Object\Container\Container $objectManager */
        $objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Object\\Container\\Container');
        /** @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager */
        $configurationManager =
            $objectManager->getInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManagerInterface');
        $extensionSettings = $parameters['default']['extensionSettings'];
        $settings =
            $configurationManager->getConfiguration(
                \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
                $parameters['default']['extensionSettings']
            );
        if (is_null($settings) && (!empty($parameters['default']['extensionSettings'])) &&
            (!empty($parameters['default']['extensionExtensionPlugin']))) {

                $settings = $configurationManager->getConfiguration(
                    \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
                    $extensionSettings,
                    $parameters['default']['extensionExtensionPlugin']
                );
            }
        $selectedValue = $fieldArray['itemFormElValue'];

        $options = '';
        // define default-Value
        if (isset($parameters['default'])) {
            $default = $parameters['default'];
            // read Settings
            $extensionSettings = $parameters['default']['extensionSettings'];
            $settings =
                $configurationManager->getConfiguration(
                    \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
                    $extensionSettings
                );
            $defaultValue = (isset($default['value']) ? $default['value'] : '');
            $defaultLabel = (isset($default['label']) ? $default['label'] : 'default');
            $defaultPrefix = (isset($default['prefix']) ? $default['prefix'] : '');
            $defaultPrefix = self::parseValueInSettings($settings, $defaultPrefix);
            $options .= self::buildOptionForTemplate($defaultPrefix, $defaultValue, $defaultLabel, $selectedValue);
        }

        // define List
        if ((isset($parameters['list'])) &&
            (is_array($parameters['list'])) &&
            (count($parameters['list']) > 0)
        ) {
            $list = $parameters['list'];
            foreach ($list as $item) {
                if (
                    (is_array($item)) &&
                    (isset($item['repository']))
                ) {
                    // Extrakt Parameter
                    $respositoryName = $item['repository'];
                    $findMethod = ((!isset($item['findMethod'])) ?
                        'findAll' :
                        $item['findMethod']
                    );
                    $fieldName = self::constructGetterMethod($item, 'fieldName');
                    $fieldUniqueId = self::constructGetterMethod($item, 'fieldUniqueId');
                    $repository = $objectManager->getInstance($respositoryName);
                    if (isset($item['findParams']) ) {
                        $parts = $repository->$findMethod($item['findParams']);
                    } else {
                        $parts = $repository->$findMethod();
                    }

                    if (isset($item['extensionSettings']) && (!empty($item['extensionSettings']))) {
                       if ((isset($item['extensionExtensionPlugin'])) &&(!empty($item['extensionExtensionPlugin']))) {
                           $localSettings = $configurationManager->getConfiguration(
                               \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT,
                               $extensionSettings,
                               $parameters['default']['extensionExtensionPlugin']
                           );

                       } else {
                           $localSettings =
                               $configurationManager->getConfiguration(
                                   \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
                                   $item['extensionSettings']
                               );
                       }


                    } else {
                        $localSettings = $settings;
                    }

                    $itemPrefix = (isset($item['prefix']) ? $item['prefix'] : '');
                    $itemPrefix = self::parseValueInSettings($localSettings, $itemPrefix);

                    foreach ($parts as $part) {
                        $partLabel = $part->$fieldName();
                        $partValue = $part->$fieldUniqueId();

                        $options .= self::buildOptionForTemplate($itemPrefix, $partValue, $partLabel, $selectedValue);
                    }
                }
            }
        }

        // Build Select
        $formName = $fieldArray['itemFormElName'];
        $formId = $fieldArray['itemFormElID'];
        $select = self::buildSelectForTemplate($formName, $formId, $options);

        // build general field
        return $frontend = self::wrapFieldTemplate($select);
    }

}
