<?php
namespace Porth\Borderstyle\ViewHelpers;


/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Dr. Dieter Porth <info@mobger.de>
 *
 *  All rights reserved
 *
 *  This script is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Fluid\Core\Rendering\RenderingContextInterface;
use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
use TYPO3\CMS\Fluid\Core\ViewHelper\Facets\CompilableInterface;

/**
 * Space Removal ViewHelper
 *
 * Removes redundant spaces between HTML tags while
 * preserving the whitespace that may be inside HTML
 * tags. Trims the final result before output.
 *
 * Heavily inspired by Twig's corresponding node type.
 *
 * Status "html"
 * <code title="porth:compress with defaultStatus 'html'">
 * <porth:compress>
 * <div>
 *     <div>
 *         <div>text
 *
 * text</div>
 *     </div>
 * </div>
 * </porth:compress>
 * <output>
 * <div><div><div>text
 *
 * text</div></div></div>
 * </output>
 * Status "css"
 * <code title="porth:compress with status 'spaceless'">
 * <porth:compress>
 * <div>
 *     <div>
 *         <div>text;
 *
 * text</div>
 *     </div>
 * </div>
 * </code>
 * <output>
 * <div><div><div>text;text</div></div></div>
 * </output>
 * Status "htmlline"
 * <code title="porth:compress with defaultStatus 'css'">
 * <porth:compress>
 * <div>
 *     <div>
 *         <div>text; }
 *
 * text</div>
 *     </div>
 * </div>
 * </code>
 * <output>
 * <div> <div> <div> text;}text</div> </div> </div>
 * </output>
 */
class CompressViewHelper extends AbstractViewHelper implements CompilableInterface
{
    /**
     * @return string
     */
    /**
     * @param string $status
     * @return string
     */
    public function render($status ='html')
    {
        return static::renderStatic(['status' => $status], $this->buildRenderChildrenClosure(), $this->renderingContext);
    }

    /**
     * @param array $arguments
     * @param \Closure $childClosure
     * @param RenderingContextInterface $renderingContext
     * @return string
     */
    public static function renderStatic(array $arguments, \Closure $childClosure, RenderingContextInterface $renderingContext)
    {
        switch ($arguments['status']) {
            case 'linebreaks':
                $compressed = trim(preg_replace('/(\\s)+(\\n)/', '$2', $childClosure()));
                break;
            case 'compactspace':
                $compressed = trim(preg_replace('/(\\s+)\\s/', ' ', $childClosure()));
                break;
            case 'html':
                $compressed = trim(preg_replace('/\\>(\\s+=\\</', '><', $childClosure()));
                break;
            case 'css':
                $compressed = trim(preg_replace('/(\\s+)\\s/', ' ', $childClosure()));
                $compressed = trim(preg_replace('/([:;}\\(\\)]) /', '$1', $compressed));
                break;
            case 'spaceless':
                $compressed = trim(preg_replace('/\\s+/', '', $childClosure()));
                break;
            default :
                $compressed = trim(preg_replace('/\\>\\s+\\</', '><', $childClosure()));
                break;
        }
        return $compressed;
    }
}
